/**
  ******************************************************************************
  * @file    uac2_descriptor.h
  * @author  MK
  * @version V1.0
  * @date    01/01/2021
  * @brief   Header for uac2_descriptor.c
  ******************************************************************************
  */ 

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __UAC2_DESCRIPTOR_H
#define __UAC2_DESCRIPTOR_H

#include "uac2.h"

#define CFG_I2S_OUT			0
#define CFG_I2S_OUT_16BIT	0
#define CFG_I2S_OUT_24BIT	0
#define CFG_I2S_OUT_32BIT	0
#define CFG_I2S_IN			1
#define CFG_I2S_IN_16BIT	0
#define CFG_I2S_IN_24BIT	0
#define CFG_I2S_IN_32BIT	1

#define OUT_CS_AC_SIZE2(b) OUT_CS_AC_SIZE_##b
#define OUT_CS_AC_SIZE(b) OUT_CS_AC_SIZE2(b)
#define OUT_CS_AC_SIZE_1 UAC2_OUTPUT_CS_AC_ITF_DESC_SIZE
#define OUT_CS_AC_SIZE_0 0

#define IN_CS_AC_SIZE2(b) IN_CS_AC_SIZE_##b
#define IN_CS_AC_SIZE(b) IN_CS_AC_SIZE2(b)
#define IN_CS_AC_SIZE_1 UAC2_INPUT_CS_AC_ITF_DESC_SIZE
#define IN_CS_AC_SIZE_0 0

#define OUT_ALT_SIZE2(b) OUT_ALT_SIZE_##b
#define OUT_ALT_SIZE(b) OUT_ALT_SIZE2(b)
#define OUT_ALT_SIZE_1 UAC2_OUTPUT_ALT_SETTING_DESC_SIZE
#define OUT_ALT_SIZE_0 0

#define IN_ALT_SIZE2(b) IN_ALT_SIZE_##b
#define IN_ALT_SIZE(b) IN_ALT_SIZE2(b)
#define IN_ALT_SIZE_1 UAC2_INPUT_ALT_SETTING_DESC_SIZE
#define IN_ALT_SIZE_0 0

#define OUT_ITF_DESC_SIZE2(b) OUT_ITF_DESC_SIZE_##b
#define OUT_ITF_DESC_SIZE(b) OUT_ITF_DESC_SIZE2(b)
#define OUT_ITF_DESC_SIZE_1 UAC2_OUTPUT_ITF_DESC_SIZE
#define OUT_ITF_DESC_SIZE_0 0

#define IN_ITF_DESC_SIZE2(b) IN_ITF_DESC_SIZE_##b
#define IN_ITF_DESC_SIZE(b) IN_ITF_DESC_SIZE2(b)
#define IN_ITF_DESC_SIZE_1 UAC2_INPUT_ITF_DESC_SIZE
#define IN_ITF_DESC_SIZE_0 0

#define UAC2_CONFIG_DESC_SIZE                 (USB_LEN_CFG_DESC+UAC2_STD_IAD_SIZE+UAC2_INTERFACE_DESC_SIZE+UAC2_CS_AC_INTERFACE_DESC_SIZE+\
											   OUT_ITF_DESC_SIZE(CFG_I2S_OUT)+\
											   IN_ITF_DESC_SIZE(CFG_I2S_IN))

#define UAC2_OUTPUT_ITF_DESC_SIZE             (UAC2_CLK_SRC_DESC_SIZE+\
											   UAC2_INPUT_TERMINAL_DESC_SIZE+0x12+UAC2_OUTPUT_TERMINAL_DESC_SIZE+\
											   UAC2_STD_AS_INTERFACE_DESC_SIZE+\
											   OUT_ALT_SIZE(CFG_I2S_OUT_16BIT)+\
											   OUT_ALT_SIZE(CFG_I2S_OUT_24BIT)+\
											   OUT_ALT_SIZE(CFG_I2S_OUT_32BIT))

#define UAC2_INPUT_ITF_DESC_SIZE			  (UAC2_CLK_SRC_DESC_SIZE+\
											   UAC2_INPUT_TERMINAL_DESC_SIZE+UAC2_OUTPUT_TERMINAL_DESC_SIZE+\
											   UAC2_STD_AS_INTERFACE_DESC_SIZE+\
											   IN_ALT_SIZE(CFG_I2S_IN_16BIT)+\
											   IN_ALT_SIZE(CFG_I2S_IN_24BIT)+\
											   IN_ALT_SIZE(CFG_I2S_IN_32BIT))

#define UAC2_OUTPUT_ALT_SETTING_DESC_SIZE     (UAC2_STD_AS_INTERFACE_DESC_SIZE+UAC2_CS_AS_INTERFACE_DESC_SIZE+\
											   UAC2_FORMAT_TYPE_I_DESC_SIZE+UAC2_STANDARD_AS_ISO_EP_DESC_SIZE+UAC2_CS_AS_ISO_EP_DESC_SIZE+\
											   UAC2_STANDARD_AS_ISO_FB_EP_DESC_SIZE)

#define UAC2_INPUT_ALT_SETTING_DESC_SIZE      (UAC2_STD_AS_INTERFACE_DESC_SIZE+UAC2_CS_AS_INTERFACE_DESC_SIZE+UAC2_FORMAT_TYPE_I_DESC_SIZE+\
											   UAC2_STANDARD_AS_ISO_EP_DESC_SIZE+UAC2_CS_AS_ISO_EP_DESC_SIZE)

#define UAC2_OUTPUT_CS_AC_ITF_DESC_SIZE       (UAC2_CLK_SRC_DESC_SIZE+UAC2_INPUT_TERMINAL_DESC_SIZE+\
											   0x12+UAC2_OUTPUT_TERMINAL_DESC_SIZE)

#define UAC2_INPUT_CS_AC_ITF_DESC_SIZE        (UAC2_CLK_SRC_DESC_SIZE+UAC2_INPUT_TERMINAL_DESC_SIZE+UAC2_OUTPUT_TERMINAL_DESC_SIZE)

#define UAC2_INTERFACE_DESC_SIZE              0x09
#define UAC2_CS_AC_INTERFACE_DESC_SIZE        0x09
#define UAC2_STD_AS_INTERFACE_DESC_SIZE       0x09
#define UAC2_CS_AS_INTERFACE_DESC_SIZE        0x10
#define UAC2_STANDARD_AS_ISO_FB_EP_DESC_SIZE  0x07
#define UAC2_STANDARD_AS_ISO_EP_DESC_SIZE     0x07
#define UAC2_CS_AS_ISO_EP_DESC_SIZE           0x08
#define UAC2_INPUT_TERMINAL_DESC_SIZE         0x11
#define UAC2_OUTPUT_TERMINAL_DESC_SIZE        0x0c
#define UAC2_STD_IAD_SIZE                     0x08
#define UAC2_CLK_SRC_DESC_SIZE                0x08
#define UAC2_FORMAT_TYPE_I_DESC_SIZE          0x06
#define UAC2_CS_AC_INTERFACE_TOTAL_LENGTH     (UAC2_CS_AC_INTERFACE_DESC_SIZE+OUT_CS_AC_SIZE(CFG_I2S_OUT)+IN_CS_AC_SIZE(CFG_I2S_IN))

// Unit numbers are arbitrary selected
#define UAC2_ENTITY_CLOCK_OUT           0x04
#define UAC2_ENTITY_CLOCK_IN            0x05
// Speaker path
#define UAC2_ENTITY_SPK_INPUT_TERMINAL  0x01
#define UAC2_ENTITY_SPK_FEATURE_UNIT    0x02
#define UAC2_ENTITY_SPK_OUTPUT_TERMINAL 0x03
// Microphone path
#define UAC2_ENTITY_MIC_INPUT_TERMINAL  0x11
#define UAC2_ENTITY_MIC_OUTPUT_TERMINAL 0x13

#ifdef USE_USB_FS
#define IN_SAMPLES_PER_PACKET(freq)	((uint32_t)(freq / 1000L))
#else
#define IN_SAMPLES_PER_PACKET(freq)	((uint32_t)(freq / 8000L))
#endif

enum
{
	OUT_ALT_SETTING_CTRL = 0,
#if (CFG_I2S_OUT_16BIT == 1)
	OUT_ALT_SETTING_16BIT,
#endif
#if (CFG_I2S_OUT_24BIT == 1)
	OUT_ALT_SETTING_24BIT,
#endif
#if (CFG_I2S_OUT_32BIT == 1)
	OUT_ALT_SETTING_32BIT
#endif
};

enum
{
	IN_ALT_SETTING_CTRL = 0,
#if (CFG_I2S_IN_16BIT == 1)
	IN_ALT_SETTING_16BIT,
#endif
#if (CFG_I2S_IN_24BIT == 1)
	IN_ALT_SETTING_24BIT,
#endif
#if (CFG_I2S_IN_32BIT == 1)
	IN_ALT_SETTING_32BIT
#endif
};

extern uint8_t UAC2_CfgDesc[UAC2_CONFIG_DESC_SIZE];

#endif /* __UAC2_DESCRIPTOR_H */
