/**
  ******************************************************************************
  * @file    usb_util.h
  * @author  MK
  * @version V0.1
  * @date    08/01/2021
  * @brief   This file provides macros for handling USB registers.
  *
  ******************************************************************************
  */

#ifndef __USB_UTIL
#define __USB_UTIL

#ifdef __cplusplus
 extern "C" {
#endif

#include "stm32f7xx_ll_usb.h"

#ifdef USE_USB_FS_INTO_HS
#define USB_OTG_BASE  USB_OTG_HS
#else
#ifdef USE_USB_FS
#define USB_OTG_BASE  USB_OTG_FS
#endif 
#ifdef USE_USB_HS
#define USB_OTG_BASE  USB_OTG_HS
#endif
#endif

#define USB_DIEPCTL(epnum)	((USB_OTG_INEndpointTypeDef *)((uint32_t)USB_OTG_BASE + USB_OTG_IN_ENDPOINT_BASE + (epnum)*USB_OTG_EP_REG_SIZE))->DIEPCTL

#define USB_DOEPCTL(epnum)	((USB_OTG_OUTEndpointTypeDef *)((uint32_t)USB_OTG_BASE + USB_OTG_OUT_ENDPOINT_BASE + (epnum)*USB_OTG_EP_REG_SIZE))->DOEPCTL

#define USB_SOF_NUMBER()	((((USB_OTG_DeviceTypeDef *)((uint32_t )USB_OTG_BASE + USB_OTG_DEVICE_BASE))->DSTS&USB_OTG_DSTS_FNSOF)>>USB_OTG_DSTS_FNSOF_Pos)

#define USB_CLEAR_INCOMPLETE_IN_EP(epnum)	USB_DIEPCTL(epnum) |= (USB_OTG_DIEPCTL_EPDIS | USB_OTG_DIEPCTL_SNAK);

#define FLIP_PARITY(epnum)	if((USB_DIEPCTL(epnum) & USB_OTG_DIEPCTL_EONUM_DPID) != 0){ \
								USB_DIEPCTL(epnum) |= USB_OTG_DIEPCTL_SD0PID_SEVNFRM; \
							} \
							else { \
								USB_DIEPCTL(epnum) |= USB_OTG_DIEPCTL_SODDFRM; \
							};

#ifdef __cplusplus
}
#endif

#endif  /* __USB_UTIL */
